package com.danitheskunk.skunkstuff.Blocks;

import net.minecraft.core.BlockPos;
import net.minecraft.core.NonNullList;
import net.minecraft.network.chat.Component;
import net.minecraft.world.ContainerHelper;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.block.entity.BaseContainerBlockEntity;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.entity.ShulkerBoxBlockEntity;
import net.minecraft.world.level.block.state.BlockState;

import java.util.stream.IntStream;

public class BlockEntityMailbox extends BaseContainerBlockEntity {
	private static final int[] SLOTS = IntStream.range(0, 9).toArray();
	private NonNullList<ItemStack> itemStacks;

	public BlockEntityMailbox(BlockPos pos, BlockState state) {
		super(BlockEntityTypes.MAILBOX, pos, state);
		this.itemStacks = NonNullList.withSize(SLOTS.length, ItemStack.EMPTY);
	}

	@Override
	protected Component getDefaultName() {
		return Component.translatable("block.skunkstuff.mailbox");
	}

	@Override
	protected AbstractContainerMenu createMenu(int syncId,
		Inventory playerInventory) {
		return new MenuMailbox(syncId, playerInventory, this);
	}

	@Override
	public int getContainerSize() {
		return SLOTS.length;
	}

	@Override
	public boolean isEmpty() {
		return itemStacks.isEmpty();
	}

	@Override
	public ItemStack getItem(int slot) {
		return itemStacks.get(slot);
	}

	@Override
	public ItemStack removeItem(int slot, int amount) {
		var itemStack = ContainerHelper.removeItem(itemStacks, slot, amount);
		if(!itemStack.isEmpty()) {
			this.setChanged();
		}
		return itemStack;
	}

	@Override
	public ItemStack removeItemNoUpdate(int slot) {
		return ContainerHelper.takeItem(itemStacks, slot);
	}

	@Override
	public void setItem(int slot, ItemStack stack) {
		itemStacks.set(slot, stack);
		if(stack.getCount() > this.getMaxStackSize()) {
			stack.setCount(this.getMaxStackSize());
		}

		this.setChanged();
	}

	@Override
	public boolean stillValid(Player player) {
		if(this.level.getBlockEntity(this.worldPosition) != this) {
			return false;
		} else {
			return !(
				player.distanceToSqr(
					(double) this.worldPosition.getX() +
						0.5,
					(double) this.worldPosition.getY() + 0.5,
					(double) this.worldPosition.getZ() + 0.5
				) > 64.0
			);
		}
	}

	@Override
	public void clearContent() {
		itemStacks.clear();
	}


}
