package com.danitheskunk.skunkstuff.Blocks;

import net.minecraft.core.BlockPos;
import net.minecraft.stats.Stats;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.context.BlockPlaceContext;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.*;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.HopperBlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.phys.shapes.VoxelShape;
import org.jetbrains.annotations.Nullable;

public class BlockMailbox extends BaseEntityBlock {
	private static final VoxelShape AABB_EASTWEST = Block.box(0.0,
		0.0,
		4.0,
		16.0,
		11.0,
		12.0
	);

	private static final VoxelShape AABB_NORTHSOUTH = Block.box(4.0,
		0.0,
		0.0,
		12.0,
		11.0,
		16.0
	);
	private static final VoxelShape AABB_NOPE = Block.box(0.0,
		0.0,
		0.0,
		0.0,
		0.0,
		0.0
	);

	public BlockMailbox(Properties properties) {
		super(properties);
	}

	@Override
	public VoxelShape getShape(BlockState state, BlockGetter world,
		BlockPos pos, CollisionContext context) {
		return switch(state.getValue(DirectionalBlock.FACING)) {
			case NORTH, SOUTH -> AABB_NORTHSOUTH;
			case WEST, EAST -> AABB_EASTWEST;
			case UP, DOWN -> AABB_NOPE;
		};
	}

	protected void createBlockStateDefinition(StateDefinition.Builder<Block,
		BlockState> builder) {
		builder.add(DirectionalBlock.FACING);
	}

	public BlockState getStateForPlacement(BlockPlaceContext ctx) {
		return (BlockState) this.defaultBlockState().setValue(
			DirectionalBlock.FACING,
			ctx.getNearestLookingDirection().getOpposite()
		);
	}

	@Nullable
	@Override
	public BlockEntity newBlockEntity(BlockPos pos, BlockState state) {
		return new BlockEntityMailbox(pos, state);
	}

	@Override
	public RenderShape getRenderShape(BlockState state) {
		return RenderShape.MODEL;
	}

	public InteractionResult use(BlockState state, Level world, BlockPos pos, Player player, InteractionHand hand, BlockHitResult hit) {
		if (world.isClientSide) {
			return InteractionResult.SUCCESS;
		} else {
			BlockEntity blockEntity = world.getBlockEntity(pos);
			if (blockEntity instanceof BlockEntityMailbox) {
				player.openMenu((BlockEntityMailbox)blockEntity);
			}

			return InteractionResult.CONSUME;
		}
	}
}
